/**
  @file
  @brief    FreeRTOS port for iHOME.
  @version  1.0
  @date     02-Sep-15
*/
/*
 * =============================================================================
 * (C) Copyright 2016, Future Technology Devices International Ltd.
 * =============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * =============================================================================
 * Modification History
 * v1.0 @ 02-Sep-15
 * 1. Creation.
 */
/*
    FreeRTOS V8.2.2 - Copyright (C) 2015 Real Time Engineers Ltd.
    All rights reserved

    VISIT http://www.FreeRTOS.org TO ENSURE YOU ARE USING THE LATEST VERSION.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation >>!AND MODIFIED BY!<< the FreeRTOS exception.

    ***************************************************************************
    >>!   NOTE: The modification to the GPL is included to allow you to     !<<
    >>!   distribute a combined work that includes FreeRTOS without being   !<<
    >>!   obliged to provide the source code for proprietary components     !<<
    >>!   outside of the FreeRTOS kernel.                                   !<<
    ***************************************************************************

    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  Full license text is available on the following
    link: http://www.freertos.org/a00114.html

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS provides completely free yet professionally developed,    *
     *    robust, strictly quality controlled, supported, and cross          *
     *    platform software that is more than just the market leader, it     *
     *    is the industry's de facto standard.                               *
     *                                                                       *
     *    Help yourself get started quickly while simultaneously helping     *
     *    to support the FreeRTOS project by purchasing a FreeRTOS           *
     *    tutorial book, reference manual, or both:                          *
     *    http://www.FreeRTOS.org/Documentation                              *
     *                                                                       *
    ***************************************************************************

    http://www.FreeRTOS.org/FAQHelp.html - Having a problem?  Start by reading
    the FAQ page "My application does not run, what could be wrong?".  Have you
    defined configASSERT()?

    http://www.FreeRTOS.org/support - In return for receiving this top quality
    embedded software for free we request you assist our global community by
    participating in the support forum.

    http://www.FreeRTOS.org/training - Investing in training allows your team to
    be as productive as possible as early as possible.  Now you can receive
    FreeRTOS training directly from Richard Barry, CEO of Real Time Engineers
    Ltd, and the world's leading authority on the world's leading RTOS.

    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool, a DOS
    compatible FAT file system, and our tiny thread aware UDP/IP stack.

    http://www.FreeRTOS.org/labs - Where new FreeRTOS products go to incubate.
    Come and try FreeRTOS+TCP, our new open source TCP/IP stack for FreeRTOS.

    http://www.OpenRTOS.com - Real Time Engineers ltd. license FreeRTOS to High
    Integrity Systems ltd. to sell under the OpenRTOS brand.  Low cost OpenRTOS
    licenses offer ticketed support, indemnification and commercial middleware.

    http://www.SafeRTOS.com - High Integrity Systems also provide a safety
    engineered and independently SIL3 certified version for use in safety and
    mission critical applications that require provable dependability.

    1 tab == 4 spaces!
*/

#include "stdio.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/* Debug. */
#define DPRINTF             DPrintf

/*==============================================================================
 * These are trace APIs.
 *
 * \see vTaskSwitchContext() for more information.
 *==============================================================================
 */
/*
 * Task control block.  A task control block (TCB) is allocated for each task,
 * and stores task state information, including a pointer to the task's context
 * (the task's run time environment, including register values)
 *
 * \note
 * Replicated from tasks.c.
 * Always counter-check when migrating to newer version of FreeRTOS.
 */
/* Value that can be assigned to the eNotifyState member of the TCB. */
typedef enum
{
    eNotWaitingNotification = 0,
    eWaitingNotification,
    eNotified
} eNotifyValue;

typedef struct tskTaskControlBlock
{
    volatile StackType_t    *pxTopOfStack;  /*< Points to the location of the last item placed on the tasks stack.  THIS MUST BE THE FIRST MEMBER OF THE TCB STRUCT. */

    #if ( portUSING_MPU_WRAPPERS == 1 )
        xMPU_SETTINGS   xMPUSettings;       /*< The MPU settings are defined as part of the port layer.  THIS MUST BE THE SECOND MEMBER OF THE TCB STRUCT. */
        BaseType_t      xUsingStaticallyAllocatedStack; /* Set to pdTRUE if the stack is a statically allocated array, and pdFALSE if the stack is dynamically allocated. */
    #endif

    ListItem_t          xGenericListItem;   /*< The list that the state list item of a task is reference from denotes the state of that task (Ready, Blocked, Suspended ). */
    ListItem_t          xEventListItem;     /*< Used to reference a task from an event list. */
    UBaseType_t         uxPriority;         /*< The priority of the task.  0 is the lowest priority. */
    StackType_t         *pxStack;           /*< Points to the start of the stack. */
    char                pcTaskName[ configMAX_TASK_NAME_LEN ];/*< Descriptive name given to the task when created.  Facilitates debugging only. */ /*lint !e971 Unqualified char types are allowed for strings and single characters only. */

    #if ( portSTACK_GROWTH > 0 )
        StackType_t     *pxEndOfStack;      /*< Points to the end of the stack on architectures where the stack grows up from low memory. */
    #endif

    #if ( portCRITICAL_NESTING_IN_TCB == 1 )
        UBaseType_t     uxCriticalNesting;  /*< Holds the critical section nesting depth for ports that do not maintain their own count in the port layer. */
    #endif

    #if ( configUSE_TRACE_FACILITY == 1 )
        UBaseType_t     uxTCBNumber;        /*< Stores a number that increments each time a TCB is created.  It allows debuggers to determine when a task has been deleted and then recreated. */
        UBaseType_t     uxTaskNumber;       /*< Stores a number specifically for use by third party trace code. */
    #endif

    #if ( configUSE_MUTEXES == 1 )
        UBaseType_t     uxBasePriority;     /*< The priority last assigned to the task - used by the priority inheritance mechanism. */
        UBaseType_t     uxMutexesHeld;
    #endif

    #if ( configUSE_APPLICATION_TASK_TAG == 1 )
        TaskHookFunction_t pxTaskTag;
    #endif

    #if( configNUM_THREAD_LOCAL_STORAGE_POINTERS > 0 )
        void *pvThreadLocalStoragePointers[ configNUM_THREAD_LOCAL_STORAGE_POINTERS ];
    #endif

    #if ( configGENERATE_RUN_TIME_STATS == 1 )
        uint32_t        ulRunTimeCounter;   /*< Stores the amount of time the task has spent in the Running state. */
    #endif

    #if ( configUSE_NEWLIB_REENTRANT == 1 )
        /* Allocate a Newlib reent structure that is specific to this task.
        Note Newlib support has been included by popular demand, but is not
        used by the FreeRTOS maintainers themselves.  FreeRTOS is not
        responsible for resulting newlib operation.  User must be familiar with
        newlib and must provide system-wide implementations of the necessary
        stubs. Be warned that (at the time of writing) the current newlib design
        implements a system-wide malloc() that must be provided with locks. */
        struct  _reent xNewLib_reent;
    #endif

    #if ( configUSE_TASK_NOTIFICATIONS == 1 )
        volatile uint32_t ulNotifiedValue;
        volatile eNotifyValue eNotifyState;
    #endif

#ifdef FT32_PORT
    StackType_t         *pxStackBase;       /*< Points to the base of the stack. */
#endif
} tskTCB;

typedef tskTCB TCB_t;

extern volatile TCB_t * volatile pxCurrentTCB;

static void vPortTaskContextPrint(volatile StackType_t *pxTopOfStack, int type)
{
    int i;
    DPRINTF("\r\n\t%s\r\n", type ? "Base" : "Current");
    for(i=0; i<30; i++)
    {
        DPRINTF("\t[R_%x]   = %x\r\n", 30-i, (int)pxTopOfStack[i]);
    }
    DPRINTF("\t[R_0,arg] = %x\r\n", (int)pxTopOfStack[30]);
    DPRINTF("\t[ret]    = %x\r\n", (int)pxTopOfStack[31]);
    for(i=32; i<=portCONTEXT_SIZE; i++)
    {
        DPRINTF("\t[xxx]   = %x\r\n", (int)pxTopOfStack[i]);
    }
    DPRINTF("\r\n");
}

/*
 * Normal functioning stack should be
 *
 * <High Address>
 * --- max --- pxCurrentTCB->pxStackBase
 *
 * --- cur --- pxCurrentTCB->pxTopOfStack
 *
 * --- min --- pxCurrentTCB->pxStack
 * <Low Address>
 *
 */
static void vPortTaskInfoPrint(const char dir)
{
    DPRINTF("Task Switch %c\r\n", dir);
    DPRINTF("\t%s stack\r\n", pxCurrentTCB->pcTaskName);
    DPRINTF("\tSaved SP = %x\r\n", (int)pxCurrentTCB->pxTopOfStack);
    DPRINTF("\tLimit    = %x\r\n", (int)pxCurrentTCB->pxStack);
    DPRINTF("\tCaller   = %x\r\n", lDebugContextSwitchCaller);
}

static void vPortTaskStackPrint(void)
{
    StackType_t *base = pxCurrentTCB->pxStackBase;
    volatile StackType_t *top  = pxCurrentTCB->pxStack < pxCurrentTCB->pxTopOfStack ? pxCurrentTCB->pxStack : pxCurrentTCB->pxTopOfStack;
    for(volatile StackType_t *i=top; i<=base; i++)
    {
        DPRINTF("\t[%x] = %x\r\n", (int)i, *i);
    }
}

/*
 * kahho::
 * vPortTaskSwitched_AssertIrqIssue() below is used to facilitate checking against the unexpected IRQ issue.
 */
inline void vPortTaskSwitched_AssertIrqIssue(void)
{
    if((lDebugFlagIntMaskState == portSTATE_INT_DISABLED && portIRQ_IS_NOT(IRQ_MASK_ALL))
            || (lDebugFlagIntMaskState == portSTATE_INT_ENABLED && portIRQ_IS_NOT(IRQ_UNMASK_ALL)))
    {
        DPRINTF("ERROR [%d] %s(): %s lDebugFlagIntMaskState = %x, IRQ_CTRL = %x\r\n",
                ulPortSysTickGet(),
                __FUNCTION__,
                pxCurrentTCB->pcTaskName,
                lDebugFlagIntMaskState,
                INTERRUPT->IRQ_CTRL);
    }
}

void vPortTaskSwitched(char dir)
{
    vPortTaskInfoPrint(dir);

    /* Check state of current context. */
    vPortTaskContextPrint(pxCurrentTCB->pxTopOfStack, 0);

    /* Check that the guard zone is not corrupted by adjacent task. */
    vPortTaskContextPrint(pxCurrentTCB->pxStackBase - portCONTEXT_SIZE, 1);

    vPortTaskSwitched_AssertIrqIssue();
}

inline void vPortTaskSwitched_Short(char dir)
{
    DPRINTF("[%d] %c %s %x %x %x %x %x %x %d\r\n",
            ulPortSysTickGet(),
            dir,
            pxCurrentTCB->pcTaskName,
            lDebugContextSwitchCaller,
            lDebugFlagIntMaskState,
            lDebugISRReturnAddr,
            pxCurrentTCB->pxStackBase,
            pxCurrentTCB->pxTopOfStack,
            pxCurrentTCB->pxStack,
            pxCurrentTCB->uxCriticalNesting);

    vPortTaskSwitched_AssertIrqIssue();
}

inline void vPortTaskCreate(void *pxNewTCB)
{
    TCB_t *p = (TCB_t *)pxNewTCB;
    DPRINTF("C %s %x %x %x\r\n",
            p->pcTaskName,
            p->pxStackBase,
            p->pxTopOfStack,
            p->pxStack);
}

inline void vPortTaskCreateFailed(void)
{
    DPRINTF("xTaskCreate(): failed\r\n");
}

inline void vPortTaskDelay(void)
{
    DPRINTF("vPortTaskDelay(): %s\r\n", pxCurrentTCB->pcTaskName);
}

int lDebugEnterCriticalSection_Caller;
inline void vPortAssert_vTaskEnterCriticalInISR(void)
{
    if(lDebugIsInISR == portENTER_ISR)
    {
        DPRINTF("ERROR [%d] %s(): Task = %s, Caller = %x\r\n",
                ulPortSysTickGet(),
                __FUNCTION__,
                pxCurrentTCB->pcTaskName,
                lDebugEnterCriticalSection_Caller);
        while(1);
    }
}

/*==============================================================================
 *
 *==============================================================================
 */
#if (configUSE_IDLE_HOOK == 1)
/*
 * See http://www.freertos.org/RTOS-idle-task.html.
 */
static int gIdleHookCount = 0;
void vApplicationIdleHook(void )
{
    //DPRINTF("vApplicationIdleHook() %d\r\n", gIdleHookCount++);
    Sys_UpdateMBCounter();
}
#endif //configUSE_IDLE_HOOK

#if (configCHECK_FOR_STACK_OVERFLOW > 0)
/*
 * See http://www.freertos.org/Stacks-and-stack-overflow-checking.html.
 *
 * We disable all interrupts and loop forever here for ease of debugging.
 */
void vApplicationStackOverflowHook(TaskHandle_t xTask, signed char *pcTaskName)
{
    portDISABLE_INTERRUPTS();
    DPRINTF("vApplicationStackOverflowHook(): stack overflow in task \"%s\"\r\n", pcTaskName);
    vPortTaskInfoPrint('S');
    /* Check state of current context. */
    vPortTaskContextPrint(pxCurrentTCB->pxTopOfStack, 0);
    vPortTaskStackPrint();
    while(1);
}
#endif  //configCHECK_FOR_STACK_OVERFLOW

#if (configUSE_MALLOC_FAILED_HOOK == 1)
/*
 * See http://www.freertos.org/a00110.html#configUSE_MALLOC_FAILED_HOOK.
 */
void vApplicationMallocFailedHook(void)
{
    DPRINTF("vApplicationMallocFailedHook(): malloc failed in task \"%s\"\r\n", pxCurrentTCB->pcTaskName);
}
#endif //configUSE_MALLOC_FAILED_HOOK
